@@ -1,9 +1,8 @@
+use once_cell::sync::OnceCell;
+
 use crate::{
-    player::Player,
-    playlist, stream,
-    video::{self, Video},
-    youtube::{innertube::Api, ytcfg::YtCfg},
-    Playlist, Stream,
+    channel, player::Player, playlist, stream, video, youtube::innertube::Api, Channel, Playlist,
+    Stream, Video,
 };
 
 use std::sync::Arc;
@@ -11,8 +10,8 @@ use std::sync::Arc;
 /// A Client capable of interacting with YouTube
 #[derive(Debug)]
 pub struct Client {
-    pub(crate) client: reqwest::Client,
-    pub(crate) player: Player,
+    pub(crate) http: reqwest::Client,
+    pub(crate) player: OnceCell<Player>,
     pub(crate) api: Api,
 }
 
@@ -20,29 +19,29 @@ impl Client {
     /// Create a new [`Client`]
     pub async fn new() -> crate::Result<Arc<Self>> {
         let http = reqwest::Client::new();
-        let home = http
-            .get("https://youtube.com/?hl=en")
-            .send()
-            .await?
-            .error_for_status()?;
-
-        let body = home.text().await?;
-
-        let (_, ytcfg) = lazy_regex::regex_captures!(r"\nytcfg.set\((\{.*\})\);", &body)
-            .expect("YoutubeConfig was unable to be found");
-        let ytcfg: YtCfg = serde_json::from_str(ytcfg).expect("YoutubeConfig was not valid json");
-
-        let player = Player::from_url(&http, &ytcfg.player_js_url)
-            .await
-            .expect("Unable to parse player");
-
         Ok(Arc::new(Client {
-            player,
-            api: Api::new(http.clone(), ytcfg),
-            client: http,
+            player: OnceCell::new(),
+            api: Api::new(http.clone()).await?,
+            http,
         }))
     }
 
+    pub(crate) async fn init_player(&self) {
+        if self.player.get().is_none() {
+            let player = Player::from_url(&self.http, self.api.config.js_url())
+                .await
+                .expect("Unable to parse player");
+
+            if self.player.set(player).is_err() {
+                log::warn!("Reinitialized player. Oh well...");
+            }
+        }
+    }
+
+    pub(crate) fn player(&self) -> &Player {
+        self.player.get().expect("Player not initialized!")
+    }
+
     /// Get a [`Video`] identified by a [`Id`](video::Id)
     pub async fn video(self: &Arc<Self>, id: video::Id) -> crate::Result<Video> {
         Video::get(Arc::clone(self), id).await
@@ -60,4 +59,9 @@ impl Client {
     pub async fn playlist(self: &Arc<Self>, id: playlist::Id) -> crate::Result<Playlist> {
         Playlist::get(Arc::clone(self), id).await
     }
+
+    /// Get a [`Channel`] identified by a [`Id`](channel::Id)
+    pub async fn channel(self: &Arc<Self>, id: channel::Id) -> crate::Result<Channel> {
+        Channel::get(Arc::clone(self), id).await
+    }
 }
