@@ -3,7 +3,7 @@
 //! # Example
 //!
 //! ```rust
-//! # #[tokio::main] async fn main() -> Result<(), Box<dyn std::error::Error>> {
+//! # #[async_std::main] async fn main() -> Result<(), Box<dyn std::error::Error>> {
 //! let client = ytextract::Client::new().await?;
 //!
 //! let streams = client.streams("nI2e-J6fsuk".parse()?).await?;
@@ -25,60 +25,33 @@ pub use self::common::Stream as Common;
 pub use self::video::Stream as Video;
 pub use crate::youtube::player_response::Quality;
 use crate::{
-    youtube::{
-        player_response::{FormatType, PlayabilityErrorCode, StreamingData},
-        video_info::VideoInfo,
-    },
+    youtube::player_response::{FormatType, StreamingData},
     Client,
 };
 use reqwest::Url;
 use std::{collections::HashMap, sync::Arc};
 
-/// A Error that can occur when working with [`Stream`]s
-#[derive(Debug, thiserror::Error)]
-pub enum Error {
-    /// Unable to get the content-length of a [`Stream`]
-    #[error("Unable to get content-length")]
-    UnknownContentLength,
-
-    /// Streams are unplayable due to a YouTube error
-    #[error("{code:?}: '{reason:?}'")]
-    Unplayable {
-        /// The [`PlayabilityErrorCode`] returned by YouTube for processing
-        code: PlayabilityErrorCode,
-        /// The optional Human-readable reason for the error
-        reason: Option<String>,
-    },
-}
-
 pub(crate) async fn get(
     client: Arc<Client>,
     id: crate::video::Id,
     streaming_data: Option<StreamingData>,
 ) -> crate::Result<impl Iterator<Item = Stream>> {
     let streaming_data = if let Some(streaming_data) = streaming_data {
-        Ok(streaming_data)
+        streaming_data
     } else {
-        let video_info = VideoInfo::from_id(&client.client, id).await?;
+        client.api.streams(id).await?.into_std()?
+    };
 
-        let player_response = video_info.player_response();
-        if player_response
-            .playability_status
-            .status
-            .is_stream_recoverable()
-        {
-            Ok(player_response
-                .streaming_data
-                .expect("Recoverable error did not contain streaming data"))
-        } else {
-            Err(Error::Unplayable {
-                code: player_response.playability_status.status,
-                reason: player_response.playability_status.reason,
-            })
-        }
-    }?;
+    let needs_player = streaming_data
+        .adaptive_formats
+        .iter()
+        .any(|f| f.base.url.is_none());
 
-    // FIXME: DashManifest/HlsManifest
+    if needs_player {
+        client.init_player().await;
+    }
+
+    // TODO: DashManifest/HlsManifest
     Ok(streaming_data
         .adaptive_formats
         .into_iter()
@@ -133,7 +106,7 @@ impl Stream {
                     serde_urlencoded::from_str(signature_cipher.as_str())
                         .expect("signatureCipher was not urlencoded");
 
-                let signature = client.player.cipher().run(root["s"].clone());
+                let signature = client.player().cipher().run(root["s"].clone());
                 let signature_arg = &root["sp"];
                 let mut url = Url::parse(&root["url"])
                     .expect("signatureCipher url attribute was not a valid URL");
@@ -190,32 +163,3 @@ impl std::fmt::Debug for Stream {
         Ok(())
     }
 }
-
-#[cfg(test)]
-mod test {
-    #[tokio::test]
-    async fn get() -> Result<(), Box<dyn std::error::Error>> {
-        let client = crate::Client::new().await?;
-
-        let mut streams = client
-            .streams("https://www.youtube.com/watch?v=7B2PIVSWtJA".parse()?)
-            .await?;
-
-        assert!(streams.next().is_some());
-
-        Ok(())
-    }
-
-    #[tokio::test]
-    async fn age_restricted() -> Result<(), Box<dyn std::error::Error>> {
-        let client = crate::Client::new().await?;
-
-        let mut streams = client
-            .streams("https://www.youtube.com/watch?v=9Jg_Fwc0QOY".parse()?)
-            .await?;
-
-        assert!(streams.next().is_some());
-
-        Ok(())
-    }
-}
