@@ -1,7 +1,6 @@
 use crate::youtube::player_response::AudioFormat;
 
 /// A [`Stream`](super::Stream) specifically containing audio data.
-#[derive(Debug)]
 pub struct Stream {
     pub(super) common: super::common::Stream,
     pub(super) audio: AudioFormat,
@@ -25,4 +24,18 @@ impl Stream {
     pub fn channels(&self) -> u64 {
         self.audio.audio_channels
     }
+
+    pub(super) fn debug(&self, debug: &mut std::fmt::DebugStruct) {
+        debug
+            .field("sample_rate", &self.sample_rate())
+            .field("channels", &self.channels());
+    }
+}
+
+impl std::fmt::Debug for Stream {
+    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+        let mut debug = f.debug_struct("AudioStream");
+        self.debug(&mut debug);
+        debug.finish()
+    }
 }
