@@ -28,7 +28,7 @@ impl Stream {
         } else {
             let res = self
                 .client
-                .client
+                .http
                 .head(self.url())
                 .send()
                 .await?
@@ -36,7 +36,7 @@ impl Stream {
 
             Ok(res
                 .content_length()
-                .ok_or(super::Error::UnknownContentLength)?)
+                .expect("HEAD request did not have a content-length"))
         }
     }
 
@@ -46,7 +46,7 @@ impl Stream {
     ) -> crate::Result<impl futures_core::Stream<Item = Result<bytes::Bytes, reqwest::Error>>> {
         Ok(self
             .client
-            .client
+            .http
             .get(self.url())
             .send()
             .await?
