@@ -4,12 +4,57 @@ use chrono::{DateTime, Utc};
 use reqwest::Url;
 use serde::Deserialize;
 
+#[derive(Debug, Deserialize)]
+#[serde(rename_all = "camelCase", untagged)]
+pub enum StreamResult {
+    Ok {
+        #[serde(rename = "streamingData")]
+        streaming_data: StreamingData,
+    },
+    Error {
+        #[serde(rename = "playabilityStatus")]
+        playability_status: PlayabilityStatus,
+    },
+}
+
+impl StreamResult {
+    pub fn into_std(self) -> crate::Result<StreamingData> {
+        match self {
+            Self::Error { playability_status } => {
+                Err(crate::Error::Youtube(playability_status.as_error()))
+            }
+            Self::Ok { streaming_data } => Ok(streaming_data),
+        }
+    }
+}
+
+#[derive(Debug, Deserialize)]
+#[serde(rename_all = "camelCase", untagged)]
+pub enum Result {
+    Ok(PlayerResponse),
+    Error {
+        #[serde(rename = "playabilityStatus")]
+        playability_status: PlayabilityStatus,
+    },
+}
+
+impl Result {
+    pub fn into_std(self) -> crate::Result<PlayerResponse> {
+        match self {
+            Self::Error { playability_status } => {
+                Err(crate::Error::Youtube(playability_status.as_error()))
+            }
+            Self::Ok(ok) => Ok(ok),
+        }
+    }
+}
+
 #[derive(Debug, Deserialize)]
 #[serde(rename_all = "camelCase")]
 pub struct PlayerResponse {
     pub streaming_data: Option<StreamingData>,
-    pub video_details: Option<VideoDetails>,
-    pub microformat: Option<Microformat>,
+    pub video_details: VideoDetails,
+    pub microformat: Microformat,
     pub playability_status: PlayabilityStatus,
 }
 
@@ -18,14 +63,12 @@ pub struct PlayerResponse {
 #[serde(rename_all = "camelCase")]
 pub struct VideoDetails {
     pub title: String,
-    #[serde_as(as = "serde_with::DisplayFromStr")]
     pub video_id: crate::video::Id,
     #[serde_as(as = "serde_with::DurationSeconds<String>")]
     pub length_seconds: Duration,
 
     #[serde(default)]
     pub keywords: Vec<String>,
-    #[serde_as(as = "serde_with::DisplayFromStr")]
     pub channel_id: crate::channel::Id,
 
     pub author: String,
@@ -87,7 +130,6 @@ pub struct Format {
 #[derive(Debug, Deserialize, Clone)]
 #[serde(rename_all = "camelCase")]
 pub struct CommonFormat {
-    // common:
     #[serde_as(as = "Option<serde_with::DisplayFromStr>")]
     #[serde(default)]
     pub url: Option<Url>,
@@ -124,7 +166,6 @@ pub struct VideoFormat {
     pub width: u64,
     pub height: u64,
     pub fps: u64,
-    pub color_info: Option<ColorInfo>,
     pub quality_label: String,
 }
 
@@ -139,54 +180,6 @@ pub struct AudioFormat {
     pub audio_channels: u64,
 }
 
-#[serde_with::serde_as]
-#[derive(Debug, serde::Deserialize, Clone)]
-#[serde(rename_all = "camelCase")]
-pub struct ColorInfo {
-    #[serde_as(as = "Option<serde_with::DisplayFromStr>")]
-    #[serde(default)]
-    pub matrix_coefficients: Option<ColorType>,
-
-    #[serde_as(as = "Option<serde_with::DisplayFromStr>")]
-    #[serde(default)]
-    pub primaries: Option<ColorType>,
-
-    #[serde_as(as = "serde_with::DisplayFromStr")]
-    pub transfer_characteristics: ColorType,
-}
-
-#[derive(Debug, Clone)]
-pub enum ColorType {
-    /// [BT709](https://en.wikipedia.org/wiki/Rec._709) color space
-    BT709,
-}
-
-/// A Error that can occur during [`ColorType`] parsing
-#[derive(Debug, thiserror::Error)]
-#[error("Unknown ColorType '{0}'")]
-pub struct ColorTypeError(String);
-
-impl std::str::FromStr for ColorType {
-    type Err = ColorTypeError;
-
-    fn from_str(s: &str) -> Result<Self, Self::Err> {
-        const PREFIXES: &[&str] = &[
-            "COLOR_MATRIX_COEFFICIENTS_",
-            "COLOR_PRIMARIES_",
-            "COLOR_TRANSFER_CHARACTERISTICS_",
-        ];
-        let ty = PREFIXES
-            .iter()
-            .find_map(|pr| s.strip_prefix(pr))
-            .ok_or_else(|| ColorTypeError(s.to_string()))?;
-
-        Ok(match ty {
-            "BT709" => ColorType::BT709,
-            _ => return Err(ColorTypeError(ty.to_string())),
-        })
-    }
-}
-
 #[serde_with::serde_as]
 #[derive(Debug, serde::Deserialize, Clone)]
 #[serde(rename_all = "camelCase")]
@@ -217,46 +210,54 @@ pub enum Quality {
 #[derive(Debug, serde::Deserialize)]
 #[serde(rename_all = "camelCase")]
 pub struct PlayabilityStatus {
-    pub status: PlayabilityErrorCode,
-    pub reason: Option<String>,
+    pub status: String,
+    pub error_screen: Option<ErrorScreen>,
 }
 
-// The doc(hidden) error codes are not actually errors or are handled by us, but
-// because this enum is `#[non_exhaustive]` this is fine
-/// A error-code returned by YouTube when a [`Video`](crate::Video) is unplayable
-#[derive(Debug, serde::Deserialize, PartialEq, Eq, Clone, Copy)]
-#[serde(rename_all = "SCREAMING_SNAKE_CASE")]
-#[non_exhaustive]
-pub enum PlayabilityErrorCode {
-    #[doc(hidden)]
-    Ok,
-    #[doc(hidden)]
-    AgeVerificationRequired,
-    #[doc(hidden)]
-    LoginRequired,
+impl PlayabilityStatus {
+    fn as_error(&self) -> crate::error::Youtube {
+        use crate::error::Youtube;
 
-    /// A livestream is currently offline
-    LiveStreamOffline,
+        let err = self
+            .error_screen
+            .as_ref()
+            .expect("Error did not have an Error screen")
+            .player_error_message_renderer
+            .as_ref();
 
-    /// A Generic Error
-    Error,
+        let err = if let Some(err) = err {
+            err
+        } else {
+            return Youtube::PurchaseRequired;
+        };
 
-    /// Unplayable
-    Unplayable,
-}
-
-impl PlayabilityErrorCode {
-    pub(crate) fn is_recoverable(self) -> bool {
-        matches!(
-            self,
-            Self::Ok
-                | Self::AgeVerificationRequired
-                | Self::LoginRequired
-                | Self::LiveStreamOffline
-                | Self::Unplayable
-        )
-    }
-    pub(crate) fn is_stream_recoverable(self) -> bool {
-        matches!(self, Self::Ok)
+        match err.reason.simple_text.as_str() {
+            "Video unavailable" => Youtube::NotFound,
+            "Private video" => Youtube::Private,
+            "This video has been removed for violating YouTube's Community Guidelines." => {
+                Youtube::CommunityGuidelineViolation
+            }
+            "Sign in to confirm your age" => Youtube::AgeRestricted,
+            e => unimplemented!("Unknown error screen text: '{}'", e),
+        }
     }
 }
+
+#[derive(Debug, serde::Deserialize)]
+#[serde(rename_all = "camelCase")]
+pub struct ErrorScreen {
+    pub player_error_message_renderer: Option<PlayerErrorMessageRenderer>,
+}
+
+#[derive(Debug, serde::Deserialize)]
+#[serde(rename_all = "camelCase")]
+pub struct PlayerErrorMessageRenderer {
+    pub reason: SimpleText,
+    //pub subreason: Option<SimpleText>,
+}
+
+#[derive(Debug, serde::Deserialize)]
+#[serde(rename_all = "camelCase")]
+pub struct SimpleText {
+    pub simple_text: String,
+}
